// $Id: util_list.h,v 1.2 2000/02/13 07:35:30 mian Exp $

/*
	TODO: Re-write our own list handler using only what we need


   Copyright (C) Gorep Development
   All rights reserved.

   Created by: Per Ghosh (e-mail: hulken@algonet.se).

   Sweden

FAQ:
   Q: How can i create a string class.
   A: CBuffer<char,'0'> bufferText;
      bufferText = "This is a text.";
      bufferText += " Some more text";
      int iPosition = bufferText.Find( "Some" );

   Q: How can I store some integer values but none of them has the value -1
   A: CBuffer<int,-1> bufferInt;

   Q: How can I store some pointers
   A: CBuffer<void*,NULL> bufferPtr;

   Q: I want to store some objects (CMyObject) in a array.
   A: int iDummy;
      CMyObject* pMyObject;
      CArrayPtr<CMyObject> arrayToMyObject;
      arrayToMyObject.Add( new CMyObject() );
      iDummy = 10;
      arrayToMyObject.Add( new CMyObject( iDummy ) );
      iDummy = 20;
      arrayToMyObject.InsertAt( 1, new CMyObject( iDummy ) );
      pMyObject = arrayToMyObject.GetAt( 0 );
      iDummy = arrayToMyObject.GetSize(); // gets the value 3
      arrayToMyObject.DeleteItems();

   Q: I want to store CXxx in a hash
   A: CMapPtr<ULONG,CXxx> mapptrToXxx;
      mapptrToXxx.InitHashTable( 21 );
      mapptrToXxx.SetAt( 10, new CXxx );
      CXxx* pXxx;
      if( mapptrToXxx.Lookup( 10, &pXxx ) == true ) { ... }
      mapptrToXxx.Empty( true );
*/

#if !defined(AFX_STACK_H__C6CFF427_5D38_11D2_8158_46FADD000000__INCLUDED_)
#define AFX_STACK_H__C6CFF427_5D38_11D2_8158_46FADD000000__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#ifndef _INC_NEW
	#include <new.h>
#endif


template<class ITEM>
void CreateItems( ITEM* pItems, UINT uCount ) {
	memset( (void*)pItems, 0, uCount * sizeof( ITEM ) );
   while( uCount != 0 )
   {
      uCount--;
      // Don't allocate, just construct
      ::new((void*)pItems) ITEM;
      pItems++;
   }
}

template<class ITEM>
void DestroyItems( ITEM* pItems, UINT uCount ) {
   while( uCount != 0 )
   {
      uCount--;
      pItems->~ITEM();
      pItems++;
   }
}

template<class ITEM>
void CopyItems(ITEM* pitemTo, const ITEM* pitemFrom, UINT uCount ) {
   while( uCount != 0 )
   {
      *pitemTo = *pitemFrom;
      pitemTo++;
      pitemFrom++;
   }
}


// ************************************************************************************************
template<class ITEM>
class CStackPtr  
{
public:
	CStackPtr( UINT uCount = 1 );
	virtual ~CStackPtr();

// Get/Set
public:
   // Get number of elements on stack
   UINT GetCount();
   // Set how many more openings that should be created if we add more then can fit
   void SetGrowBy( UINT uGrowBy );

// Operation
public:
   // Add pointer to stack
   void Push( ITEM* pITEM );
   // Remove pointer from stack
   ITEM* Pop();
   // Delete items in stack          
   void DeleteItems();

// Attributes
private:
   UINT m_uCount;    // Items in stack
   UINT m_uMaxCount; // max number of items before we need to realocate
   UINT m_uGrowBy;   // how many more valuew that should fit if we need to allocate more space for stack

   ITEM** m_ppITEM;  // Pointer to pointers in stack
};                   

template<class ITEM>
CStackPtr<ITEM>::CStackPtr( UINT uCount ) {
   m_uCount    = 0;
   m_uMaxCount = 0;
   m_uGrowBy   = uCount;
   m_ppITEM    = NULL;
}

template<class ITEM>
inline CStackPtr<ITEM>::~CStackPtr() {
   delete m_ppITEM;
}

template<class ITEM>
inline UINT CStackPtr<ITEM>::GetCount() {
   return m_uCount; }

template<class ITEM>
inline void CStackPtr<ITEM>::SetGrowBy( UINT uGrowBy ) {
   m_uGrowBy = uGrowBy;
}

template<class ITEM>
void CStackPtr<ITEM>::Push( ITEM* pITEM ) {
   ITEM** ppITEM;
   if( m_uCount >= m_uMaxCount )
   {
      m_uMaxCount += m_uGrowBy;
      ppITEM = new ITEM*[m_uMaxCount];
      memcpy( ppITEM, m_ppITEM, sizeof(ITEM*) * m_uCount );
      delete m_ppITEM;
      m_ppITEM = ppITEM;
   }

   m_ppITEM[m_uCount] = pITEM;
   m_uCount++;
}

template<class ITEM>
ITEM* CStackPtr<ITEM>::Pop() {
   m_uCount--;
   return m_ppITEM[m_uCount];
}

template<class ITEM>
void CStackPtr<ITEM>::DeleteItems() {
   while( m_uCount )
   {
      m_uCount--;
      delete m_ppITEM[m_uCount];
   }
}



// ****************************************************************************
/* CArrayPtr is a tiny wrapper for storing pointers in a array. It can also
   delete that objects that are stored but it does not delete pointer's when
   array object is destroyed. The object pointers that are stored is not copied
   and that means that if you insert a pointer to array and you modify it later
   the object that is pointed to in array is also modified.
*/
template<class ITEM>
class CArrayPtr
{
// Construction
public: 
   CArrayPtr( UINT uCount = 1 );
   ~CArrayPtr();

	ITEM* operator[](UINT uIndex) const;

// Get/Set
public:
   // Get number of elements on stack
   UINT GetSize();
   // Set how many more openings that should be created if we add more then can fit
   void SetGrowBy( UINT uGrowBy );

// Operation
public:
   // Add item to array
   void  Add( ITEM* pITEM );
   // Insert item at index
   void  InsertAt( UINT uIndex, ITEM* pITEM );
   // Get item for index
   ITEM* GetAt( UINT uIndex ) const;
   // Set item at index
   ITEM* SetAt( UINT uIndex, ITEM* pITEM );
   // Switch position for two items in array
   void  SwitchItems( UINT uIndex1, UINT uIndex2 );
   // find
   UINT Find(ITEM* pITEM);
   // remove item at index
   ITEM* RemoveAt( UINT uIndex );
   void  Remove( UINT uStart, UINT uCount, bool bDelete );
   // delete all items (make sure that all items is allocated with new !)
   void  DeleteItems();

protected:
   void SafeSize( UINT uSlots );
                         
// Attributes
private:
   UINT m_uCount;    // Items in array
   UINT m_uMaxCount; // max number of items before we need to realocate
   UINT m_uGrowBy;   // how many more valuew that should fit if we need to allocate more space for stack

   ITEM** m_ppITEM;  // Pointer to pointers in stack
};

template<class ITEM>
CArrayPtr<ITEM>::CArrayPtr( UINT uCount ) {
   m_uCount    = 0;
   m_uMaxCount = 0;
   m_uGrowBy   = uCount;
   m_ppITEM    = NULL;
}

template<class ITEM>
inline CArrayPtr<ITEM>::~CArrayPtr() {
   delete m_ppITEM;
}

template<class ITEM>
inline ITEM* CArrayPtr<ITEM>::operator[](UINT uIndex) const	{ 
   return GetAt( uIndex ); }


template<class ITEM>
inline UINT CArrayPtr<ITEM>::GetSize() {
   return m_uCount; }


template<class ITEM>
inline void CArrayPtr<ITEM>::SetGrowBy( UINT uGrowBy ) {
   m_uGrowBy = uGrowBy;
}


template<class ITEM>
void CArrayPtr<ITEM>::Add( ITEM* pITEM ) {
   SafeSize( m_uCount + 1 );
   m_ppITEM[m_uCount] = pITEM;
   m_uCount++;
}

template<class ITEM>
UINT CArrayPtr<ITEM>::Find(ITEM* pITEM ) {
   UINT uCount;
   UINT uIndex;
   uCount = m_uCount;
   for( uIndex = 0; uIndex < uCount; uIndex++ )
   {
      if( m_ppITEM[uIndex] == pITEM ) return uIndex;
   }
	return -1;
}

template<class ITEM>
void CArrayPtr<ITEM>::InsertAt( UINT uIndex, ITEM* pITEM ) {
   SafeSize( m_uCount + 1 );
   memmove( &m_ppITEM[uIndex+1], &m_ppITEM[uIndex], sizeof(ITEM*) * (m_uCount - uIndex) );
   m_ppITEM[uIndex] = pITEM;
   m_uCount++;
}

template<class ITEM>
inline ITEM* CArrayPtr<ITEM>::GetAt( UINT uIndex ) const {
   return m_ppITEM[uIndex];
}

template<class ITEM>
ITEM* CArrayPtr<ITEM>::SetAt( UINT uIndex, ITEM* pITEM ) {
   ITEM* pIReturn;
   pIReturn = m_ppITEM[uIndex];
   m_ppITEM[uIndex] = pITEM;
   return pIReturn;
}

template<class ITEM>
void CArrayPtr<ITEM>::SwitchItems( UINT uIndex1, UINT uIndex2 ) {
   ITEM* pTemp;
   pTemp = m_ppITEM[uIndex1];
   m_ppITEM[uIndex1] = m_ppITEM[uIndex2];
   m_ppITEM[uIndex2] = pTemp;
}

template<class ITEM>
ITEM* CArrayPtr<ITEM>::RemoveAt( UINT uIndex ) {
   ITEM* pIReturn;
   pIReturn = m_ppITEM[uIndex];
   memmove( &m_ppITEM[uIndex], &m_ppITEM[uIndex+1], sizeof(ITEM*) * (m_uCount - uIndex) );
   m_uCount--;
   return pIReturn;
}

template<class ITEM>
void CArrayPtr<ITEM>::Remove( UINT uStart, UINT uCount, bool bDelete ) {
   UINT uCounter;
   UINT uEnd;
   uEnd = uStart + uCount;
   if( uEnd > m_uCount ) uEnd = m_uCount;
   if( bDelete == true )
   {
      for( uCounter = uStart; uCounter < uEnd; uCounter++ )
      {
         delete m_ppITEM[uCounter];
      }
   }
   memmove( &m_ppITEM[uStart], &m_ppITEM[uEnd-1], sizeof(ITEM*) * (m_uCount - uEnd) );
   m_uCount -= (uEnd - uStart);
}

template<class ITEM>
void CArrayPtr<ITEM>::DeleteItems() {
   while( m_uCount )
   {
      m_uCount--;
      delete m_ppITEM[m_uCount];
   }
}


template<class ITEM>
void CArrayPtr<ITEM>::SafeSize( UINT uSlots ) {
   ITEM** ppITEM;
   if( uSlots >= m_uMaxCount )
   {
      m_uMaxCount += m_uGrowBy + uSlots;
      ppITEM = new ITEM*[m_uMaxCount];
      memcpy( ppITEM, m_ppITEM, sizeof(ITEM*) * m_uCount );
      delete m_ppITEM;
      m_ppITEM = ppITEM;
   }
}



// ****************************************************************************
/* CBuffer is a template class that can be used to save a specified type in a
   buffer. It has functionality for working with the buffer. You can for 
   example use it for mangage text (char's or short's).
*/
template<class TYPE, TYPE end>
class CBuffer
{
public:
   CBuffer(); // Default Constructor
   CBuffer( UINT uGrowBy ); // Sets the number of items that buffer is going to grow when it needs to allocate more
   CBuffer( TYPE* pTYPE ); // Constructor that takes data and inserts it
   CBuffer( TYPE* pTYPE, UINT uMaxLength ); // Constructor that takes data and inserts it, but not more data than max length
   CBuffer( const CBuffer& other ); // Copy contructor
   ~CBuffer();

   // Operators
   const CBuffer& operator=(const CBuffer& other); 
   operator const TYPE*();
	TYPE operator[](UINT uIndex) const;
   const CBuffer& operator+=(TYPE type); 
   const CBuffer& operator+=(TYPE* pTYPE); 

   // Copy object
   void Copy( const CBuffer& other );

   // Get size for buffer (how many items)
   UINT GetSize();
   // Get size for the internal buffer (items that could fit before new allocation is done)
   UINT GetBufferSize();
   // Set how much array should grow if reallocation is done
   void SetGrowBy( UINT uGrow );
   // Sets the length for buffer (WARNING!!! Be sure that you know what you are doing)
   void SetLength( UINT uLength );
   // If buffer isn't as long as size the buffer is increased
   void SafeBufferSize( UINT uSize );

   // Add item to buffer (inserted last)
   void  Add( TYPE type );
   // Add items to buffer (inserted last)
   void  Add( TYPE* pTYPE );
   // Get item at specified index
   TYPE  GetAt( UINT uIndex ) const;
   // Set type at specified position
   void  SetAt( UINT uIndex, TYPE type );
   // Insert type at specified index
   void  InsertAt( UINT uIndex, TYPE type );
   // Remove item for specified index
   void  RemoveAt( UINT uIndex );
   // Find item in buffer and return index to it
   int   Find( TYPE type );
   int   Find( TYPE* pTYPE );
   // Return pointer to buffer that holds all data
   TYPE* Data();
   //
   void  ReleaseBuffer();
   // Grows the buffer from current length and returns last position
   TYPE* Grow( UINT uLength );
   // Clear buffer (same as setting length to 0)
   void  Empty();
   // Attach data to buffer
   void  Attach( TYPE* pTYPE );
   // Detaches buffer and clear all members (doesn't delete data)
   TYPE* Detach();

protected:
   void SafeSize( UINT uLength );

// Attributes
private:
   UINT m_uCount;    // Items in array
   UINT m_uMaxCount; // max number of items before we need to realocate
   UINT m_uGrowBy;   // how many more valuew that should fit if we need to allocate more space for stack

   TYPE* m_pTYPE;    // Buffer

   static TYPE m_TEnd;
};

template<class TYPE, TYPE end>
TYPE CBuffer<TYPE,end>::m_TEnd = end;

template<class TYPE, TYPE end>
CBuffer<TYPE,end>::CBuffer() {
   memset( this, 0, sizeof(CBuffer) );
   m_pTYPE = &CBuffer<TYPE,end>::m_TEnd;
}

template<class TYPE, TYPE end>
CBuffer<TYPE,end>::CBuffer( UINT uGrowBy ) {
   m_uCount    = 0;
   m_uMaxCount = 0;
   m_uGrowBy   = uGrowBy;
   m_pTYPE     = &CBuffer<TYPE,end>::m_TEnd;
}

template<class TYPE, TYPE end>
CBuffer<TYPE,end>::CBuffer( TYPE* pTYPE ) {
   UINT uLength = 0;
   while( pTYPE[uLength] != CBuffer<TYPE,end>::m_TEnd ) uLength++;
   m_pTYPE = new TYPE[uLength];
   memcpy( m_pTYPE, pTYPE, sizeof(TYPE) * uLength );
   m_uCount = m_uMaxCount = uLength;
   m_uGrowBy = 0;
}

template<class TYPE, TYPE end>
CBuffer<TYPE,end>::CBuffer( TYPE* pTYPE, UINT uMaxLength ) {
   UINT uCheck = 0;
   while( (uCheck < uMaxLength) && (pTYPE[uCheck] != CBuffer<TYPE,end>::m_TEnd) ) uCheck++;
   if( uCheck == 0 )
   {
      m_pTYPE = &CBuffer<TYPE,end>::m_TEnd;
   }
   else
   {
      m_pTYPE = new TYPE[uCheck + 1];
      memcpy( m_pTYPE, pTYPE, sizeof(TYPE) * uCheck );
      m_pTYPE[uCheck] = CBuffer<TYPE,end>::m_TEnd;
   }
   m_uCount = m_uMaxCount = uCheck;
   m_uGrowBy = 0;
}

template<class TYPE, TYPE end>
CBuffer<TYPE,end>::CBuffer( const CBuffer<TYPE,end>& other ) {
   m_pTYPE = NULL;
   Copy( other );
}

template<class TYPE, TYPE end>
const CBuffer<TYPE,end>& CBuffer<TYPE,end>::operator=(const CBuffer& other) {
   Copy( other ); return *this;
}


template<class TYPE, TYPE end>
inline CBuffer<TYPE,end>::~CBuffer() {
   if( m_pTYPE != &CBuffer<TYPE,end>::m_TEnd ) delete m_pTYPE;
}

template<class TYPE, TYPE end>
inline CBuffer<TYPE,end>::operator const TYPE*() {
   return m_pTYPE; }

template<class TYPE, TYPE end>
inline TYPE CBuffer<TYPE,end>::operator[](UINT uIndex) const	{ 
   return GetAt( uIndex ); 
}

template<class TYPE, TYPE end>
inline const CBuffer<TYPE,end>& CBuffer<TYPE,end>::operator+=(TYPE type) {
   Add( type ); return *this;
}

template<class TYPE, TYPE end>
inline const CBuffer<TYPE,end>& CBuffer<TYPE,end>::operator+=(TYPE* pTYPE) {
   Add( pTYPE ); return *this;
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::Copy( const CBuffer<TYPE,end>& other ) {
   if( m_pTYPE != &CBuffer<TYPE,end>::m_TEnd ) delete m_pTYPE;
   m_uCount = other.m_uCount;
   m_uMaxCount = m_uCount + 1;
   m_uGrowBy = other.m_uGrowBy;
   if( m_uCount > 0 )
   {
      m_pTYPE = new TYPE[m_uMaxCount];
      memcpy( m_pTYPE, other.m_pTYPE, sizeof(TYPE) * (m_uCount + 1) );
   }
   else
   {
      m_uMaxCount = 0;
      m_pTYPE = &CBuffer<TYPE,end>::m_TEnd;
   }
}

template<class TYPE, TYPE end>
inline UINT CBuffer<TYPE,end>::GetSize() {
   return m_uCount; 
}

template<class TYPE, TYPE end>
inline UINT CBuffer<TYPE,end>::GetBufferSize() {
   return m_uMaxCount; 
}

template<class TYPE, TYPE end>
inline void CBuffer<TYPE,end>::SetGrowBy( UINT uGrow ) {
   m_uGrowBy = uGrow;
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::SetLength( UINT uLength ) {
   m_pTYPE[uLength] = CBuffer<TYPE,end>::m_TEnd;
   m_uCount = uLength;
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::SafeBufferSize( UINT uSize ) {
   if( m_uMaxCount <= uSize ) SafeSize( uSize + 1 );
}


template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::Add( TYPE type ) {
   SafeSize( m_uCount + 2 );
   m_pTYPE[m_uCount] = type;
   m_uCount++;
   m_pTYPE[m_uCount] = CBuffer<TYPE,end>::m_TEnd;
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::Add( TYPE* pTYPE ) {
   UINT uLength = 0;
   while( pTYPE[uLength++] != CBuffer<TYPE,end>::m_TEnd);
   SafeSize( uLength + m_uCount );
   memcpy( &m_pTYPE[m_uCount], pTYPE, sizeof(TYPE) * uLength );
   m_uCount += uLength - 1;
   m_pTYPE[m_uCount] = CBuffer<TYPE,end>::m_TEnd;
}

template<class TYPE, TYPE end>
inline TYPE CBuffer<TYPE,end>::GetAt( UINT uIndex ) const {
   return m_pTYPE[uIndex]; 
}

template<class TYPE, TYPE end>
inline void CBuffer<TYPE,end>::SetAt( UINT uIndex, TYPE type ) {
   m_pTYPE[uIndex] = type;
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::InsertAt( UINT uIndex, TYPE type ) {
   SafeSize( m_uCount + 1 );
   memmove( &m_pTYPE[uIndex+1], &m_pTYPE[uIndex], sizeof(TYPE) * (m_uCount - uIndex) );
   m_pTYPE[uIndex] = type;
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::RemoveAt( UINT uIndex ) {
   memmove( &m_pTYPE[uIndex], &m_pTYPE[uIndex+1], sizeof(TYPE) * (m_uCount - uIndex) );
   m_uCount--;
}

template<class TYPE, TYPE end>
int CBuffer<TYPE,end>::Find( TYPE type ) {
   UINT uCount;
   UINT uIndex;
   uCount = m_uCount;
   for( uIndex = 0; uIndex < uCount; uIndex++ )
   {
      if( m_pTYPE[uIndex] == type ) return uIndex;
   }
   return -1;
}

template<class TYPE, TYPE end>
int CBuffer<TYPE,end>::Find( TYPE* pTYPE ) {
   UINT uIndex = 0;
   UINT uCount = m_uCount;
   UINT uCheck;
   TYPE* pCheck;
   for( uIndex = 0; uIndex < uCount; uIndex++ )
   {
      if( m_pTYPE[uIndex] == pTYPE[0] )
      {
         uCheck = 1;
         pCheck = &m_pTYPE[uIndex];
         while( (pTYPE[uCheck] != CBuffer<TYPE,end>::m_TEnd) && (pCheck[uCheck] == pTYPE[uCheck]) ) uCheck++;
         if( pTYPE[uCheck] == CBuffer<TYPE,end>::m_TEnd ) return uCheck;
      }
   }
   return -1;
}

template<class TYPE, TYPE end>
inline TYPE* CBuffer<TYPE,end>::Data() {
   return m_pTYPE; 
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::ReleaseBuffer() {
   UINT uCount = 0;
   TYPE* pTYPE = m_pTYPE;
   while( pTYPE[uCount] != CBuffer<TYPE,end>::m_TEnd ) uCount++;
   m_uCount = uCount;
}

template<class TYPE, TYPE end>
inline void CBuffer<TYPE,end>::Empty() {
   SetLength( 0 );
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::Attach( TYPE* pTYPE ) {
   UINT uCount = 0;
   while( pTYPE[uCount] != end ) uCount++;
   m_uMaxCount = m_uCount = uCount;
   if( m_pTYPE != &CBuffer<TYPE,end>::m_TEnd ) delete m_pTYPE;
   m_pTYPE = pTYPE; 
}

template<class TYPE, TYPE end>
TYPE* CBuffer<TYPE,end>::Detach() {
   TYPE* pTYPE;
   pTYPE = (m_pTYPE == &CBuffer<TYPE,end>::m_TEnd) ?  NULL : m_pTYPE;
   m_uMaxCount = m_uCount = 0;
   m_pTYPE = &CBuffer<TYPE,end>::m_TEnd;
   return pTYPE;
}

template<class TYPE, TYPE end>
TYPE* CBuffer<TYPE,end>::Grow( UINT uLength ) {
   SafeSize( m_uCount + uLength );
   return &m_pTYPE[m_uCount];
}

template<class TYPE, TYPE end>
void CBuffer<TYPE,end>::SafeSize( UINT uLength ) {
   TYPE* pTYPE;
   if( uLength >= m_uMaxCount )
   {
      m_uMaxCount = m_uGrowBy + uLength;
      pTYPE = new TYPE[m_uMaxCount];
      memcpy( pTYPE, m_pTYPE, sizeof(TYPE) * m_uCount );
      if( m_pTYPE != &CBuffer<TYPE,end>::m_TEnd ) delete m_pTYPE;
      m_pTYPE = pTYPE;
   }
}



// ****************************************************************************
/* This class stores pointer in a hash table. Use this when you need to find
   a object fast and there is a lot of objects. If you only have 10 or less
   objects it is better to use a array but otherwise this class will speed up
   things. It uses a litle more memory than a array but that can be accepted
   if the speed is increased. There is very litle extra memory that is used.
*/
template<class KEY, class ITEM>
class CMapPtr
{
public: struct ITERATOR;
private: struct Entry;
public:
   CMapPtr();
   ~CMapPtr();

// get/set
public:
   UINT GetCount();

// operations
public:
   // Initializes the hashobject. This function has to be called before you can use it.
   void  InitHashTable( UINT uSize );
   // Get value at specified key
   ITEM* GetAt( KEY key );
   // Set a value at specified key
   void  SetAt( KEY key, ITEM* pITEM );
   // Remove item for specified key and returns the pointer to item that was removed
   ITEM* RemoveKey( KEY key );
   // Find item at specified key
   bool  Lookup( KEY key, ITEM** ppITEM );
   // Get first key in hash
   void  GetFirstPosition( ITERATOR& iterator );
   // Gets next item and updates key (returnes NULL if no more items)
   bool  GetNext( ITERATOR& iterator, ITEM** ppItem );
   // Remove items from hash table (if bDelete is true then the objects are deleted)
   void  Empty( bool bDelete );
   // Calculate the hash key
   UINT  HashKey( KEY key );

   // item that exists in hash object. Use this when iterating through all items in hash table
   struct ITERATOR
   {
      UINT uBranch;
      UINT uIndex;
   };

protected:
   Entry* FindKey( KEY key, Entry*, UINT uSize );
   void  Add( UINT uEntry, KEY key, ITEM* pITEM );

private:
   struct Entry
   {
      KEY   m_KEY;
      ITEM* m_pITEM;
   };
// Attributes
private:
   UINT   m_uCount;
   UINT   m_uGrowBy;    // How much a branch should grow if items are added and there is no more room
   UINT   m_uTableSize; // Table size for entries.
   UINT*  m_puTable;    // Has number if items in each entry (branch) and also the amount that can 
                        // be inserted before reallocation is done.
   Entry** m_ppEntry;   // Pointer to pointer for Entry object's
};

template<class KEY, class ITEM>
CMapPtr<KEY,ITEM>::CMapPtr() {
   m_uCount = 0; m_uGrowBy = 1; m_uTableSize = 0; m_puTable = NULL; m_ppEntry = 0; 
}

template<class KEY, class ITEM>
CMapPtr<KEY,ITEM>::~CMapPtr() {
   Empty( false );
}

template<class KEY, class ITEM>
inline UINT CMapPtr<KEY,ITEM>::GetCount() {
   return m_uCount; 
}

template<class KEY, class ITEM>
void CMapPtr<KEY,ITEM>::InitHashTable( UINT uSize ) {
   m_puTable = new UINT[uSize*2];
   m_ppEntry  = new Entry*[uSize];
   memset( m_puTable, 0, sizeof(UINT) * uSize * 2);
   memset( m_ppEntry, 0, sizeof(Entry*) * uSize );
   m_uTableSize = uSize;
}

template<class KEY, class ITEM>
ITEM* CMapPtr<KEY,ITEM>::GetAt( KEY key ) {
   UINT uIndex;
   Entry* pEntry;
   uIndex = HashKey( key );
   pEntry = FindKey( key, m_ppEntry[uIndex], m_puTable[uIndex] );
   if( pEntry != NULL )
   {
      return pEntry->m_pITEM;
   }
   return NULL;
}

template<class KEY, class ITEM>
void CMapPtr<KEY,ITEM>::SetAt( KEY key, ITEM* pITEM ) {
   UINT uIndex;
   Entry* pEntry;
   uIndex = HashKey( key );
   pEntry = FindKey( key, m_ppEntry[uIndex], m_puTable[uIndex] );
   if( pEntry == NULL ) Add( uIndex, key, pITEM );
   else 
   {
      pEntry->m_pITEM = pITEM;
   }
}

template<class KEY, class ITEM>
ITEM* CMapPtr<KEY,ITEM>::RemoveKey( KEY key ) {
   UINT uIndex;
   UINT uSize;
   Entry* pentryFind;
   Entry* pEntry;
   ITEM* pITEM;
   uIndex = HashKey( key );
   uSize  = m_puTable[uIndex];
   pEntry = m_ppEntry[uIndex];
   while( uSize != 0 )
   {
      uSize--;
      pentryFind = &pEntry[uSize];
      if( pentryFind->m_KEY == key )
      {
         pITEM = pentryFind->m_pITEM;
         m_puTable[uIndex]--;
         memmove( &pEntry[uSize], &pEntry[uSize+1], sizeof(Entry) * m_puTable[uIndex] );
         m_uCount--;
         return pITEM;
      }
   }
   return NULL;
}

template<class KEY, class ITEM>
bool CMapPtr<KEY,ITEM>::Lookup( KEY key, ITEM** ppITEM ) {
   UINT uIndex;
   Entry* pEntry;
   uIndex = HashKey( key );
   pEntry = m_ppEntry[uIndex];
   pEntry = FindKey( key, m_ppEntry[uIndex], m_puTable[uIndex] );
   if( pEntry != NULL )
   {
      *ppITEM = pEntry->m_pITEM;
      return true;
   }
   return false;
}

template<class KEY, class ITEM>
void CMapPtr<KEY,ITEM>::GetFirstPosition( ITERATOR& iterator ) {
   for( UINT uBranch = 0; uBranch < m_uTableSize; uBranch++ )
   {
      if( m_puTable[uBranch] != 0 )
      {
         iterator.uBranch = uBranch;
         iterator.uIndex = 0;
         return;
      }
   }
   iterator.uBranch = m_uTableSize + 1;
}


template<class KEY, class ITEM>
bool CMapPtr<KEY,ITEM>::GetNext( ITERATOR& iterator, ITEM** ppItem ) {
   UINT uBranch;

   uBranch = iterator.uBranch;
   if( uBranch >= m_uTableSize ) return false;

   for( ;uBranch < m_uTableSize; uBranch++ )
   {
      if( iterator.uIndex < m_puTable[uBranch] )
      {
         *ppItem = m_ppEntry[uBranch][iterator.uIndex].m_pITEM;
         iterator.uBranch = uBranch;
         iterator.uIndex++;
         return true;
      }

      iterator.uIndex = 0;
   }

   return false;
}

template<class KEY, class ITEM>
void CMapPtr<KEY,ITEM>::Empty( bool bDelete ) {
   UINT uBranch;
   UINT uIndex;
   uBranch = m_uTableSize;
   while( uBranch != 0 )
   {
      uBranch--;
      if( bDelete )
      {
         uIndex = m_puTable[uBranch];
         while( uIndex != 0 )
         {
            uIndex--;
            delete m_ppEntry[uBranch][uIndex].m_pITEM;
         }
      }
      delete m_ppEntry[uBranch];
   }
   delete m_puTable;
   m_puTable = NULL;
   delete m_ppEntry;
   m_ppEntry = NULL;
   m_uTableSize = 0;
   m_uCount = 0;
}



template<class KEY, class ITEM>
UINT CMapPtr<KEY,ITEM>::HashKey( KEY key ) {
   UINT uEntry;
   uEntry = key % m_uTableSize;
   return uEntry;
}

template<class KEY, class ITEM>
CMapPtr<KEY,ITEM>::Entry* CMapPtr<KEY,ITEM>::FindKey( KEY key, Entry* pEntry, UINT uSize ) {
   Entry* pentryFind;
   while( uSize != 0 )
   {
      uSize--;
      pentryFind = &pEntry[uSize];
      if( pentryFind->m_KEY == key ) return pentryFind;
   }
   return NULL;
}

template<class KEY, class ITEM>
void CMapPtr<KEY,ITEM>::Add( UINT uEntry, KEY key, ITEM* pITEM ) {
   UINT uSize;
   UINT uMaxSize;
   Entry* pentryTemp;
   Entry* pEntry = m_ppEntry[uEntry];
   uMaxSize      = m_puTable[uEntry+m_uTableSize];
   if( m_puTable[uEntry] >= uMaxSize )
   {
      uMaxSize += m_uGrowBy;
      pentryTemp = new Entry[uMaxSize];
      memcpy( pentryTemp, pEntry, sizeof(Entry) * m_puTable[uEntry] );
      delete pEntry ;
      pEntry = pentryTemp;
      m_puTable[uEntry+m_uTableSize] = uMaxSize;
      m_ppEntry[uEntry] = pEntry;
   }
   uSize = m_puTable[uEntry];
   pEntry[uSize].m_KEY = key;
   pEntry[uSize].m_pITEM = pITEM;
   uSize++;
   m_puTable[uEntry] = uSize;
   m_uCount++;
}

// ****************************************************************************

template<class ITEM>
class CArray
{
// Construction
public: 
   CArray( UINT uCount = 1 );
   ~CArray();

	ITEM& operator[](UINT uIndex) const;

// Get/Set
public:
   // Get number of elements on stack
   UINT GetSize();
   // Set how many more openings that should be created if we add more then can fit
   void SetGrowBy( UINT uGrowBy );

// Operation
public:
   // Add item to array
   void  Add( const ITEM& item );
   // Get item for index
   ITEM& GetAt( UINT uIndex ) const;
   // Set item at index
   void  SetAt( UINT uIndex, const ITEM& item );
   // Insert item at index
   void  InsertAt( UINT uIndex, const ITEM& item );
   // remove item at index
   void  RemoveAt( UINT uIndex );

protected:
   void SafeSize( UINT uSlots );

// Attributes
private:
   UINT m_uCount;    // Items in array
   UINT m_uMaxCount; // max number of items before we need to realocate
   UINT m_uGrowBy;   // how many more valuew that should fit if we need to allocate more space for stack

   ITEM* m_pITEM;    // Pointer to objects in array
};

template<class ITEM>
CArray<ITEM>::CArray( UINT uCount ) {
   m_uCount    = 0;
   m_uMaxCount = 0;
   m_uGrowBy   = uCount;
   m_pITEM     = NULL;
}

template<class ITEM>
CArray<ITEM>::~CArray() {
   DestroyItems<ITEM>( m_pITEM, m_uCount );
   delete [] (BYTE*)m_pITEM;
}

template<class ITEM>
inline ITEM& CArray<ITEM>::operator[](UINT uIndex) const	{ 
   return GetAt( uIndex ); }

template<class ITEM>
inline UINT CArray<ITEM>::GetSize() {
   return m_uCount; 
}

template<class ITEM>
inline void CArray<ITEM>::SetGrowBy( UINT uGrowBy ) {
   m_uGrowBy = uGrowBy;
}

template<class ITEM>
void CArray<ITEM>::Add( const ITEM& item ) {
   SafeSize( m_uCount + 1 );
   m_pITEM[m_uCount] = item;
   m_uCount++;
}

template<class ITEM>
inline ITEM& CArray<ITEM>::GetAt( UINT uIndex ) const {
   return m_pITEM[uIndex];
}

template<class ITEM>
inline void CArray<ITEM>::SetAt( UINT uIndex, const ITEM& item ) {
   m_pITEM[uIndex] = item;
}


template<class ITEM>
void CArray<ITEM>::InsertAt( UINT uIndex, const ITEM& item ) {
   SafeSize( m_uCount + 1 );
   memmove( &m_pITEM[uIndex+1], &m_pITEM[uIndex], sizeof(ITEM) * (m_uCount - uIndex) );
   CreateItems<ITEM>( &m_pITEM[uIndex], 1 );
   m_pITEM[uIndex] = item;
   m_uCount++;
}

template<class ITEM>
void CArray<ITEM>::RemoveAt( UINT uIndex ) {
   DestroyItems<ITEM>( &m_pITEM[uIndex], 1 );
   memmove( &m_pITEM[uIndex], &m_pITEM[uIndex+1], sizeof(ITEM) * (m_uCount - uIndex) );
   m_uCount--;
}

template<class ITEM>
void CArray<ITEM>::SafeSize( UINT uSlots ) {
   ITEM* pITEM;
   if( uSlots >= m_uMaxCount )
   {
      m_uMaxCount += m_uGrowBy + uSlots;
      pITEM = (ITEM*) new BYTE[sizeof(ITEM) * m_uMaxCount]; ;
      memcpy( pITEM, m_pITEM, sizeof(ITEM) * m_uCount );
      CreateItems<ITEM>( &pITEM[m_uCount], m_uMaxCount - m_uCount );
      delete (BYTE*)m_pITEM;
      m_pITEM = pITEM;
   }
}

#endif // !defined(AFX_STACK_H__C6CFF427_5D38_11D2_8158_46FADD000000__INCLUDED_)
