// $Id: DesktopObject.cpp,v 1.9 2000/02/19 15:23:00 mian Exp $

#include "stdafx.h"
#include "Core.h"
#include "DesktopObject.h"
#include "../common/gdefs.h"
#include "../common/util_safe.h"

static TCHAR lpszDesktopClass[] = L"GraphiteDesktopClass";

BOOL CALLBACK EnumMonitorsProc(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData);

/////////////////////////////////////////////////////////////////////////////
// CDesktopObject

STDMETHODIMP CDesktopObject::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IDesktopObject,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

CDesktopObject::CDesktopObject()
{
	m_hWnd = NULL;
	ZeroMemory(&m_rectScreen, sizeof(m_rectScreen));

	/*
		Register our classes and create the desktop window.
	*/

	WNDCLASS wc;
	ZeroMemory(&wc, sizeof(wc));
	wc.lpfnWndProc = CDesktopObject::ProxyWndProc;
	wc.hInstance = GetModuleHandle(NULL);
	wc.lpszClassName = lpszDesktopClass;
	wc.style = CS_DBLCLKS;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.cbWndExtra = 4;
	if (!RegisterClass(&wc))
	{
		return;
	}

	m_hWnd = CreateWindowEx(
		WS_EX_TOOLWINDOW,
		lpszDesktopClass,
		NULL,
		WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
		0, 0,
		0, 0,
		GetDesktopWindow(),
		NULL,
		GetModuleHandle(NULL),
		NULL
		);

	if (m_hWnd)
	{
		SetWindowLong(m_hWnd, G_GWL_CLASSPOINTER, (LONG)this);
		SendMessage(m_hWnd, WM_DISPLAYCHANGE, 0, 0);
		ShowWindow(m_hWnd, SW_SHOW);
	}
}

CDesktopObject::~CDesktopObject()
{
	if (m_hWnd)
	{
		DestroyWindow(m_hWnd);
		m_hWnd = NULL;
	}
	UnregisterClass(lpszDesktopClass, GetModuleHandle(NULL));
}

LRESULT WINAPI CDesktopObject::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_DISPLAYCHANGE:
			{
				GetClientRect(GetDesktopWindow(), &m_rectScreen);
				EnumDisplayMonitors(NULL, NULL, EnumMonitorsProc, (LPARAM)this);
			}
			break;

		case WM_LBUTTONUP: Fire_OnLButtonUp(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_MBUTTONUP: Fire_OnMButtonUp(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_RBUTTONUP: Fire_OnRButtonUp(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_LBUTTONDOWN: Fire_OnLButtonDown(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_MBUTTONDOWN: Fire_OnMButtonDown(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_RBUTTONDOWN: Fire_OnRButtonDown(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_LBUTTONDBLCLK: Fire_OnLButtonDblClk(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_MBUTTONDBLCLK: Fire_OnMButtonDblClk(LOWORD(lParam), HIWORD(lParam)); break;
		case WM_RBUTTONDBLCLK: Fire_OnRButtonDblClk(LOWORD(lParam), HIWORD(lParam)); break;

		case WM_ERASEBKGND: break;

		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC hDC = BeginPaint(hWnd, &ps);
				PaintDesktop(hDC);
				EndPaint(hWnd, &ps);
			}
			break;

		case WM_WINDOWPOSCHANGING:
			{
				WINDOWPOS *pos = (WINDOWPOS *)lParam;
				if (IsValidWritePtr(pos))
				{
					pos->hwndInsertAfter = HWND_BOTTOM;
				}
			}
			break;

		case WM_NCPAINT:
			{
				if (!wParam) SetWindowPos(hWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
			}
			break;

		case WM_SYSCOMMAND:
			break;

		case WM_ACTIVATE:
			{
				if (LOWORD(wParam)) SetActiveWindow(GetParent(hWnd));
			}
			break;

		default:
			return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}
	return 0;
}

LRESULT WINAPI CDesktopObject::ProxyWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CDesktopObject *p = (CDesktopObject *)GetWindowLong(hWnd, G_GWL_CLASSPOINTER);
	if (IsValidReadPtr(p)) return p->WndProc(hWnd, uMsg, wParam, lParam);
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

BOOL CALLBACK EnumMonitorsProc(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
{
	CDesktopObject *pDesktop = (CDesktopObject *)dwData;
	if (IsValidReadPtr(pDesktop))
	{
		/*
			Adjust desktop window if needed, what we end up with is a desktop that
			spans the entire multi-monitor area, we also support negative co-ords
			incase the primary monitor is in the middle and there is one to the left.
		*/
		if (lprcMonitor->left < pDesktop->m_rectScreen.left) pDesktop->m_rectScreen.left = lprcMonitor->left;
		if (lprcMonitor->top < pDesktop->m_rectScreen.top) pDesktop->m_rectScreen.top = lprcMonitor->top;
		if (lprcMonitor->right > pDesktop->m_rectScreen.right) pDesktop->m_rectScreen.right = lprcMonitor->right;
		if (lprcMonitor->bottom > pDesktop->m_rectScreen.bottom) pDesktop->m_rectScreen.bottom = lprcMonitor->bottom;
		SetWindowPos(pDesktop->m_hWnd, NULL, pDesktop->m_rectScreen.left, pDesktop->m_rectScreen.top, pDesktop->m_rectScreen.right, pDesktop->m_rectScreen.bottom, SWP_NOZORDER);
	}

	return TRUE;
}


STDMETHODIMP CDesktopObject::get_left(long *pVal)
{
	if (IsValidWritePtr(pVal)) *pVal = m_rectScreen.left;

	return S_OK;
}

STDMETHODIMP CDesktopObject::get_top(long *pVal)
{
	if (IsValidWritePtr(pVal)) *pVal = m_rectScreen.top;

	return S_OK;
}

STDMETHODIMP CDesktopObject::get_right(long *pVal)
{
	if (IsValidWritePtr(pVal)) *pVal = m_rectScreen.right;

	return S_OK;
}

STDMETHODIMP CDesktopObject::get_bottom(long *pVal)
{
	if (IsValidWritePtr(pVal)) *pVal = m_rectScreen.bottom;

	return S_OK;
}


STDMETHODIMP CDesktopObject::get_m_hWnd(long *pVal)
{
	*pVal = (long)m_hWnd;

	return S_OK;
}

/*
	$Log: DesktopObject.cpp,v $
	Revision 1.9  2000/02/19 15:23:00  mian
	
	Made G desktop a child of the real desktop
	
	Revision 1.8  2000/02/17 06:37:19  mian
	
	Added ActiveXObject to core component (use to load ActiveX controls inside any Window)
	
	Revision 1.7  2000/02/16 05:56:41  mian
	
	Cleaned up class pointer code
	
	Revision 1.6  2000/02/14 02:14:37  mian
	
	Added WM_ACTIVATEAPP to msg list
	
	Revision 1.5  2000/02/14 02:10:13  mian
	
	Fixed small Read->Write typo
	
	Revision 1.4  2000/02/13 08:04:07  mian
	
	Added log messages
	
*/
