// $Id: MenuObject.cpp,v 1.8 2000/02/17 05:02:39 mian Exp $

#include "stdafx.h"
#include "Core.h"
#include "MenuObject.h"
#include "../common/gdefs.h"
#include "../common/util_safe.h"

static TCHAR lpszMenuClass[] = L"GraphiteMenuClass";

/////////////////////////////////////////////////////////////////////////////
// CMenuObject

STDMETHODIMP CMenuObject::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IMenuObject,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

CMenuObject::CMenuObject()
{
	WNDCLASS wc;
	ZeroMemory(&wc, sizeof(wc));
	wc.lpfnWndProc = CMenuObject::ProxyWndProc;
	wc.hInstance = GetModuleHandle(NULL);
	wc.lpszClassName = lpszMenuClass;
	wc.style = CS_DBLCLKS;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = CreateSolidBrush(GetSysColor(COLOR_MENU));
	wc.cbWndExtra = 8;
	if (!RegisterClass(&wc))
	{
		return;
	}

	m_hWnd = CreateWindowEx(
		WS_EX_TOOLWINDOW,
		lpszMenuClass,
		NULL,
		WS_POPUP,
		0, 0,
		0, 0,
		NULL,
		NULL,
		GetModuleHandle(NULL),
		NULL
		);

	if (m_hWnd)
	{
		SetWindowLong(m_hWnd, G_GWL_CLASSPOINTER, (LONG)this);
		ShowWindow(m_hWnd, SW_HIDE);
	}
}

CMenuObject::~CMenuObject()
{
	if (m_hWnd)
	{
		DestroyWindow(m_hWnd);
		m_hWnd = NULL;
	}
	m_menuList.DeleteItems();
	UnregisterClass(lpszMenuClass, GetModuleHandle(NULL));
}

LRESULT WINAPI CMenuObject::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		default:
			return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}
	return 0;
}

LRESULT WINAPI CMenuObject::ProxyWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CMenuObject *p = (CMenuObject *)GetWindowLong(hWnd, G_GWL_CLASSPOINTER);
	if (IsValidReadPtr(p)) return p->WndProc(hWnd, uMsg, wParam, lParam);
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


STDMETHODIMP CMenuObject::get_CreateMenu(long *pVal)
{
	/* 
		Initialize to NULL first so the script can determine if it fails
	*/
	*pVal = 0;

	HMENU hMenu = ::CreatePopupMenu();
	if (IsMenu(hMenu))
	{
		CMenu *pMenu = new CMenu;
		if (IsValidReadPtr(pMenu))
		{
			pMenu->m_hMenu = hMenu;
			*pVal = (long)pMenu;
			m_menuList.Add(pMenu);
		}
	}

	return S_OK;
}

STDMETHODIMP CMenuObject::AppendMenu(long lMenu, short iType, long lParam, BSTR bstrItem)
{
	CMenu *pMenu = (CMenu *)lMenu;
	if (IsValidReadPtr(pMenu))
	{
		if (IsMenu(pMenu->m_hMenu))
		{
			::AppendMenu(pMenu->m_hMenu, MF_STRING, lParam, bstrItem);
		}
	}

	return S_OK;
}


STDMETHODIMP CMenuObject::TrackPopupMenu(long lMenu, long x, long y)
{
	CMenu *pMenu = (CMenu *)lMenu;
	if (IsValidReadPtr(pMenu))
	{
		if (IsMenu(pMenu->m_hMenu))
		{
			::TrackPopupMenu(pMenu->m_hMenu, TPM_LEFTBUTTON, x, y, 0, m_hWnd, NULL);
		}
	}

	return S_OK;
}

/*
	$Log: MenuObject.cpp,v $
	Revision 1.8  2000/02/17 05:02:39  mian
	
	Added shortcut item
	
	Revision 1.7  2000/02/17 04:34:11  mian
	
	Added shortcut object to core component
	
	Revision 1.6  2000/02/16 16:22:48  mian
	
	Changed menus to standard Windows menus, will make them owner drawn later
	
	Revision 1.5  2000/02/16 09:08:20  mian
	
	Added selected menu item highlighting
	
	Revision 1.4  2000/02/16 06:30:21  mian
	
	Semi-working popup menus, need to figure out howto make them act like proper
	Windows menus (close when mouse pressed outside etc)
	
	Revision 1.3  2000/02/16 05:56:41  mian
	
	Cleaned up class pointer code
	
	Revision 1.2  2000/02/16 04:54:13  mian
	
	Started on custom menus and script manipulation functions
	
	Revision 1.1  2000/02/15 16:01:12  mian
	
	Added menu object to core component
	
*/
