// $Id: ShortcutObject.cpp,v 1.4 2000/02/17 06:37:19 mian Exp $

#include "stdafx.h"
#include "Core.h"
#include "ShortcutObject.h"
#include "../common/gdefs.h"
#include "../common/util_safe.h"

static TCHAR lpszShortcutClass[] = L"GraphiteShortcutClass";

#define G_GWL_ITEMPOINTER	4

/////////////////////////////////////////////////////////////////////////////
// CShortcutObject

STDMETHODIMP CShortcutObject::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IShortcutObject,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

CShortcutObject::CShortcutObject()
{
	WNDCLASS wc;
	ZeroMemory(&wc, sizeof(wc));
	wc.lpfnWndProc = CShortcutObject::ProxyWndProc;
	wc.hInstance = GetModuleHandle(NULL);
	wc.lpszClassName = lpszShortcutClass;
	wc.style = CS_DBLCLKS;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = CreateSolidBrush(GetSysColor(COLOR_3DFACE));
	wc.cbWndExtra = 8;
	if (!RegisterClass(&wc))
	{
		return;
	}
}

CShortcutObject::~CShortcutObject()
{
	UnregisterClass(lpszShortcutClass, GetModuleHandle(NULL));
}

LRESULT WINAPI CShortcutObject::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_LBUTTONUP: Fire_OnLButtonUp((long)hWnd, LOWORD(lParam), HIWORD(lParam)); break;

		default:
			return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}
	return 0;
}

LRESULT WINAPI CShortcutObject::ProxyWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CShortcutObject *p = (CShortcutObject *)GetWindowLong(hWnd, G_GWL_CLASSPOINTER);
	if (IsValidReadPtr(p)) return p->WndProc(hWnd, uMsg, wParam, lParam);
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


STDMETHODIMP CShortcutObject::get_Create(int x, int y, int cx, int cy, long hwndParent, IShortcutItem **pVal)
{
	/* 
		Initialize to NULL so scripter knows if it fails
	*/
	*pVal = NULL;
	IUnknown *pUnk = NULL;
	HRESULT hr = CoCreateInstance(CLSID_ShortcutItem, NULL, CLSCTX_INPROC_SERVER, IID_IShortcutItem, (void**)&pUnk);
	if (SUCCEEDED(hr) && IsValidInterfacePtr(pUnk))
	{
		IShortcutItem *pItem = NULL;
		hr = pUnk->QueryInterface(IID_IShortcutItem, (void**)&pItem);
		if (SUCCEEDED(hr) && IsValidInterfacePtr(pItem))
		{
			HWND hWnd = CreateWindowEx(
				WS_EX_TOOLWINDOW,
				lpszShortcutClass,
				NULL,
				WS_POPUP | WS_VISIBLE,
				x, y,
				cx, cy,
				(HWND)hwndParent,
				NULL,
				GetModuleHandle(NULL),
				NULL
				);
			if (hWnd)
			{
				SetWindowLong(hWnd, G_GWL_CLASSPOINTER, (LONG)this);
				SetWindowLong(hWnd, G_GWL_ITEMPOINTER, (LONG)pItem);

				pItem->put_m_hWnd((long)hWnd);
				*pVal = pItem;
			}
		}
		pUnk->Release();
	}

	return S_OK;
}


/*
	$Log: ShortcutObject.cpp,v $
	Revision 1.4  2000/02/17 06:37:19  mian
	
	Added ActiveXObject to core component (use to load ActiveX controls inside any Window)
	
	Revision 1.3  2000/02/17 06:04:14  mian
	
	Fixed LoadObjects() not loading some objects, more shortcut item stuff
	
	Revision 1.2  2000/02/17 05:02:39  mian
	
	Added shortcut item
	
	Revision 1.1  2000/02/17 04:34:11  mian
	
	Added shortcut object to core component
	
*/
