// Wave.h : Declaration of the CWave

#ifndef __WAVE_H_
#define __WAVE_H_

#include "resource.h"       // main symbols
#include <atlctl.h>
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
// CWave
class ATL_NO_VTABLE CWave : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IDispatchImpl<IWave, &IID_IWave, &LIBID_DESKEFXLib>,
	public CComControl<CWave>,
	public IPersistStreamInitImpl<CWave>,
	public IOleControlImpl<CWave>,
	public IOleObjectImpl<CWave>,
	public IOleInPlaceActiveObjectImpl<CWave>,
	public IViewObjectExImpl<CWave>,
	public IOleInPlaceObjectWindowlessImpl<CWave>,
	public ISupportErrorInfo,
	public IPersistStorageImpl<CWave>,
	public ISpecifyPropertyPagesImpl<CWave>,
	public IQuickActivateImpl<CWave>,
	public IDataObjectImpl<CWave>,
	public IProvideClassInfo2Impl<&CLSID_Wave, NULL, &LIBID_DESKEFXLib>,
	public CComCoClass<CWave, &CLSID_Wave>
{
public:
	CWave()
	{
		m_bWindowOnly = TRUE;

		m_nOffset = 0;
		m_nWaves = 3;
		m_nWaveHeight = 5;
		m_nWidth = 0;
		m_nHeight = 0;
		m_hbmImage = NULL;
		m_hImageDC = NULL;
	}

	virtual ~CWave()
	{
		if (m_hImageDC) DeleteDC(m_hImageDC);
		if (m_hbmImage) DeleteObject(m_hbmImage);
	}

DECLARE_REGISTRY_RESOURCEID(IDR_WAVE)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CWave)
	COM_INTERFACE_ENTRY(IWave)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROP_MAP(CWave)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CWave)
	CHAIN_MSG_MAP(CComControl<CWave>)
	DEFAULT_REFLECTION_HANDLER()
	MESSAGE_HANDLER(WM_CREATE, OnCreate)
	MESSAGE_HANDLER(WM_TIMER, OnTimer)
	MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBkgnd)
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
	{
		static const IID* arr[] = 
		{
			&IID_IWave,
		};
		for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
		{
			if (InlineIsEqualGUID(*arr[i], riid))
				return S_OK;
		}
		return S_FALSE;
	}

// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// IWave
public:
	STDMETHOD(get_ImageHeight)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_ImageWidth)(/*[out, retval]*/ long *pVal);
	STDMETHOD(get_WaveHeight)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_WaveHeight)(/*[in]*/ short newVal);
	STDMETHOD(LoadImage)(BSTR bstrPath);
	STDMETHOD(get_NumWaves)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_NumWaves)(/*[in]*/ short newVal);
	BOOL BufferImage(BSTR lpszPath);
	void Calculate();

	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di)
	{
		if (m_hbmImage && m_hImageDC)
		{
			double nAngleRadians=(m_nOffset*3.6)/(180/M_PI_2);

			HBITMAP hbmBuf;
			HDC hBufDC;

			hBufDC = CreateCompatibleDC(di.hdcDraw);
			hbmBuf = CreateCompatibleBitmap(di.hdcDraw, m_nWidth, m_nHeight);
			SelectObject(hBufDC, hbmBuf);

			int y;
			for (y = 0; y < m_nHeight; y++)
			{
				int offset2=(max(min((int)((cos(nAngleRadians)*m_nAmplitude+y)),m_nHeight-1),0));
				BitBlt(hBufDC, 0, y, m_nWidth, 1, m_hImageDC, 0, offset2, SRCCOPY);
				nAngleRadians += m_nAngleStep;
			}

			m_nOffset += m_nWaveHeight;
			if (m_nOffset >= m_nHeight) m_nOffset = 0;
		
			BitBlt(di.hdcDraw, 0, 0, m_nWidth, m_nHeight, hBufDC, 0, 0, SRCCOPY);
	
			DeleteDC(hBufDC);
			DeleteObject(hbmBuf);
		}

		return S_OK;
	}
	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		SetTimer(1, 33);

		return 0;
	}
	LRESULT OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		InvalidateRect(NULL, FALSE);

		return 0;
	}
	LRESULT OnEraseBkgnd(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = TRUE;
		return 0;
	}

	double	m_nOffset;
	int		m_nWidth;
	int		m_nHeight;
	int		m_nWaves;
	int		m_nWaveHeight;
	double	m_nAngleStep;
	double	m_nAmplitude;
	HBITMAP m_hbmImage;
	HDC		m_hImageDC;
};

#endif //__WAVE_H_
