// $Id: startup.cpp,v 1.2 2000/02/23 08:54:07 mian Exp $

#include "stdafx.h"
#include "../common/util_safe.h"

/*
	Execute all the items in the specified CSIDL_* folder
*/
void
WINAPI
ExecuteCSIDLFolderItems(int nFolder)
{
	LPITEMIDLIST pItemID = NULL;
	IMalloc *pMalloc = NULL;
	HRESULT hr;

	/*
		Grab a pointer to the IMalloc interface
	*/
	hr = SHGetMalloc(&pMalloc);
	if (SUCCEEDED(hr))
	{
		/*
			Get the item ID list for the specified CSIDL folder
		*/
		hr = SHGetSpecialFolderLocation(NULL, nFolder, &pItemID);
		if (SUCCEEDED(hr))
		{
			TCHAR lpszStartupPath[MAX_PATH], lpszPath[MAX_PATH];
			HANDLE hFind;
			WIN32_FIND_DATA wFind;

			/*
				Convert the item ID list to a proper path
			*/
			SHGetPathFromIDList(pItemID, lpszStartupPath);
			::PathCombine(lpszPath, lpszStartupPath, L"*");
			hFind = FindFirstFile(lpszPath, &wFind);
			if (hFind != INVALID_HANDLE_VALUE)
			{
				do {
					if (wcscmp(wFind.cFileName, L".") && wcscmp(wFind.cFileName, L".."))
					{
						::PathCombine(lpszPath, lpszStartupPath, wFind.cFileName);
						/*
							Execute the item
						*/
						::ShellExecute(NULL, L"open", lpszPath, NULL, NULL, SW_SHOW);
					}
				} while (FindNextFile(hFind, &wFind));
				FindClose(hFind);
			}
			/*
				Use the IMalloc interface to free the item ID list
			*/
			pMalloc->Free(pItemID);
		}
		pMalloc->Release();
	}
}

/*
	Execute items in the startup folder
*/
void
WINAPI
ExecuteStartupFolderItems()
{
	/*
		Execute items in the normal/current user startup folder
	*/
	ExecuteCSIDLFolderItems(CSIDL_STARTUP);
	/*
		Execute items in the all users startup folder
	*/
	ExecuteCSIDLFolderItems(CSIDL_COMMON_STARTUP);
}

/*
	Execute items in the specified registry key
*/
void ExecuteRegistryItems(HKEY key, WCHAR *lpszPath)
{
	HKEY hKey;

	if (RegOpenKey(key, lpszPath, &hKey) == ERROR_SUCCESS)
	{
		WCHAR lpszName[MAX_PATH], lpszValue[MAX_PATH];
		DWORD dwNameSize, dwValueSize, dwType, dwIndex;

		for (dwIndex = 0; ; dwIndex++)
		{
			ZeroMemory(&lpszName, sizeof(lpszName));
			ZeroMemory(&lpszValue, sizeof(lpszValue));
			dwNameSize = sizeof(lpszName);
			dwValueSize = sizeof(lpszValue);
			dwType = REG_SZ;
			if (RegEnumValue(hKey, dwIndex, (LPTSTR)&lpszName, &dwNameSize, NULL, &dwType, (LPBYTE)&lpszValue, &dwValueSize) != ERROR_SUCCESS) break;
			ShellExecute(NULL, L"open", lpszValue, NULL, NULL, SW_SHOW);
		}

		RegCloseKey(hKey);
	}
}

/*
	Delete items in the specified registry key
*/
void DeleteRegistryItems(HKEY key, WCHAR *lpszPath)
{
	HKEY hKey;

	if (RegOpenKeyEx(key, lpszPath, 0, KEY_READ | KEY_SET_VALUE, &hKey) == ERROR_SUCCESS)
	{
		WCHAR lpszName[MAX_PATH], lpszValue[MAX_PATH];
		DWORD dwNameSize, dwValueSize, dwType, dwIndex;

		for (dwIndex = 0; ; dwIndex++)
		{
			ZeroMemory(&lpszName, sizeof(lpszName));
			ZeroMemory(&lpszValue, sizeof(lpszValue));
			dwNameSize = sizeof(lpszName);
			dwValueSize = sizeof(lpszValue);
			dwType = REG_SZ;
			if (RegEnumValue(hKey, dwIndex, (LPTSTR)&lpszName, &dwNameSize, NULL, &dwType, (LPBYTE)&lpszValue, &dwValueSize) != ERROR_SUCCESS) break;
			RegDeleteValue(hKey, lpszName);
		}

		RegCloseKey(hKey);
	}
}

/*
	Execute items in the registry run keys
*/
void WINAPI
ExecuteStartupRegistryItems()
{
	ExecuteRegistryItems(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run");
	ExecuteRegistryItems(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\RunOnce");
	DeleteRegistryItems(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\RunOnce");

	ExecuteRegistryItems(HKEY_CURRENT_USER, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run");
	ExecuteRegistryItems(HKEY_CURRENT_USER, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\RunOnce");
	DeleteRegistryItems(HKEY_CURRENT_USER, L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\RunOnce");
}

/*
	$Log: startup.cpp,v $
	Revision 1.2  2000/02/23 08:54:07  mian
	
	Added ability to execute registry startup items
	
	Revision 1.1  2000/02/22 04:44:09  mian
	
	Added ExecuteStartupFolderItems and ExecuteCSIDLFolderItems to gutil
	
*/