// $Id: ScriptObject.cpp,v 1.5 2000/02/17 06:04:14 mian Exp $

#include "stdafx.h"
#include "ScriptObject.h"
#include "../common/util_safe.h"
#include "../common/util_strings.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptObject::CScriptObject()
{
}

CScriptObject::~CScriptObject()
{
	UnloadAllObjects();
}

/*
	Attempt to load a scripting object.
*/
HRESULT CScriptObject::LoadObject(WCHAR *wszItem, REFCLSID clsid)
{
	HRESULT hr;
	IUnknown *pUnk = NULL;

	hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, IID_IUnknown, (void**)&pUnk);
	if (SUCCEEDED(hr))
	{
		CScriptObjectContainer *pObject = new CScriptObjectContainer;
		if (pObject == NULL)
		{
			hr = E_OUTOFMEMORY;
		} else {
			BSTR bstrItem = NULL;
			if (SUCCEEDED(bstrdup(&bstrItem, wszItem)))
			{
				/*
					Scripting object successfully loaded
				*/
				pObject->m_bstrName = bstrItem;
				pObject->m_pUnk = pUnk;
				m_objList.Add(pObject);
			}
		}
	}

	return hr;
}

HRESULT CScriptObject::UnloadAllObjects()
{
	m_objList.DeleteItems();

	return S_OK;
}

/*
	Retrieve a named objects IUnknown interface.
*/
HRESULT CScriptObject::GetObjectUnknown(BSTR bstrItem, IUnknown **pUnk)
{
	CScriptObjectContainer *pObject;
	UINT i;

	for (i = 0; i < m_objList.GetSize(); i++)
	{
		pObject = m_objList.GetAt(i);
		if (pObject)
		{
			if (!_wcsicmp(pObject->m_bstrName, bstrItem))
			{
				*pUnk = pObject->m_pUnk;

				return S_OK;
			}
		}
	}

	return E_FAIL;
}

/*
	Make all the scripting objects visible to the scripting host specified.
*/
HRESULT CScriptObject::AddNamedItems(CScriptHost *pHost)
{
	CScriptObjectContainer *pObject;
	UINT i;

	for (i = 0; i < m_objList.GetSize(); i++)
	{
		pObject = m_objList.GetAt(i);
		if (pObject)
		{
			if (IsValidReadPtr(pHost))
			{
				pHost->AddItem(pObject->m_bstrName);
			}
		}
	}

	return S_OK;
}

/*
	$Log: ScriptObject.cpp,v $
	Revision 1.5  2000/02/17 06:04:14  mian
	
	Fixed LoadObjects() not loading some objects, more shortcut item stuff
	
	Revision 1.4  2000/02/14 06:39:57  mian
	
	Changed component loading to only load those present in our registry key.
	Graphite Script Category still exists, we will use this to let the user choose
	what components they want enabled.  This way only we don't waste resources loading
	every component
	
	Revision 1.3  2000/02/13 08:04:08  mian
	
	Added log messages
	
*/